@@ -1,8 +1,6 @@
-/*
- * SPDX-License-Identifier: MIT
- *
- * Copyright (C) 2011-2012 Mathieu Desnoyers <mathieu.desnoyers@efficios.com>
- */
+// SPDX-FileCopyrightText: 2011-2012 Mathieu Desnoyers <mathieu.desnoyers@efficios.com>
+//
+// SPDX-License-Identifier: MIT
 
 #include <stdint.h>
 #include <stdio.h>
@@ -974,6 +972,7 @@ static									      \
 void lttng_ust__event_probe__##_provider##___##_name(LTTNG_UST__TP_ARGS_DATA_PROTO(_args)) \
 {									      \
 	struct lttng_ust_event_common *__event = (struct lttng_ust_event_common *) __tp_data; \
+	struct lttng_ust_channel_common *__chan_common;			      \
 	size_t __dynamic_len_idx = 0;					      \
 	const size_t __num_fields = LTTNG_UST__TP_ARRAY_SIZE(lttng_ust__event_fields___##_provider##___##_name) - 1; \
 	struct lttng_ust_probe_ctx __probe_ctx;				      \
@@ -986,28 +985,19 @@ void lttng_ust__event_probe__##_provider##___##_name(LTTNG_UST__TP_ARGS_DATA_PRO
 									      \
 	if (0)								      \
 		(void) __dynamic_len_idx;	/* don't warn if unused */    \
-	switch (__event->type) {					      \
-	case LTTNG_UST_EVENT_TYPE_RECORDER:				      \
-	{								      \
-		struct lttng_ust_event_recorder *__event_recorder = (struct lttng_ust_event_recorder *) __event->child; \
-		struct lttng_ust_channel_buffer *__chan = __event_recorder->chan; \
-		struct lttng_ust_channel_common *__chan_common = __chan->parent; \
-									      \
+	if (caa_unlikely(!CMM_ACCESS_ONCE(__event->enabled)))		      \
+		return;							      \
+	if (caa_unlikely(!LTTNG_UST_TP_RCU_LINK_TEST()))		      \
+		return;							      \
+	__chan_common = lttng_ust_get_chan_common_from_event_common(__event); \
+	if (__chan_common) {						      \
 		if (!LTTNG_UST__TP_SESSION_CHECK(session, __chan_common->session)) \
 			return;						      \
 		if (caa_unlikely(!CMM_ACCESS_ONCE(__chan_common->session->active))) \
 			return;						      \
 		if (caa_unlikely(!CMM_ACCESS_ONCE(__chan_common->enabled)))   \
 			return;						      \
-		break;							      \
 	}								      \
-	case LTTNG_UST_EVENT_TYPE_NOTIFIER:				      \
-		break;							      \
-	}								      \
-	if (caa_unlikely(!CMM_ACCESS_ONCE(__event->enabled)))		      \
-		return;							      \
-	if (caa_unlikely(!LTTNG_UST_TP_RCU_LINK_TEST()))		      \
-		return;							      \
 	__probe_ctx.struct_size = sizeof(struct lttng_ust_probe_ctx);	      \
 	__probe_ctx.ip = LTTNG_UST__TP_IP_PARAM(LTTNG_UST_TP_IP_PARAM);	      \
 	if (caa_unlikely(CMM_ACCESS_ONCE(__event->eval_filter))) {	      \
@@ -1056,6 +1046,24 @@ void lttng_ust__event_probe__##_provider##___##_name(LTTNG_UST__TP_ARGS_DATA_PRO
 				&__notif_ctx);				      \
 		break;							      \
 	}								      \
+	case LTTNG_UST_EVENT_TYPE_COUNTER:				      \
+	{								      \
+		struct lttng_ust_event_counter *__event_counter = (struct lttng_ust_event_counter *) __event->child; \
+		struct lttng_ust_event_counter_ctx __event_counter_ctx;	      \
+									      \
+		__event_counter_ctx.struct_size = sizeof(struct lttng_ust_event_counter_ctx); \
+		__event_counter_ctx.args_available = CMM_ACCESS_ONCE(__event_counter->use_args); \
+									      \
+		if (caa_unlikely(!__interpreter_stack_prepared && __event_counter_ctx.args_available)) \
+			lttng_ust__event_prepare_interpreter_stack__##_provider##___##_name(__stackvar.__interpreter_stack_data, \
+				LTTNG_UST__TP_ARGS_DATA_VAR(_args));	      \
+									      \
+		(void) __event_counter->chan->ops->counter_hit(__event_counter, \
+				__stackvar.__interpreter_stack_data,	      \
+				&__probe_ctx,				      \
+				&__event_counter_ctx);			      \
+		break;							      \
+	}								      \
 	}								      \
 }
 
@@ -1177,7 +1185,7 @@ static const struct lttng_ust_event_desc lttng_ust__event_desc___##_provider##_#
 #define LTTNG_UST__TRACEPOINT_EVENT_INSTANCE(_template_provider, _template_name, _provider, _name, _args) \
 	&lttng_ust__event_desc___##_provider##_##_name,
 
-static const struct lttng_ust_event_desc * const LTTNG_UST__TP_COMBINE_TOKENS(lttng_ust__event_desc___, LTTNG_UST_TRACEPOINT_PROVIDER)[] = {
+static const struct lttng_ust_event_desc * const LTTNG_UST__TP_COMBINE_TOKENS(lttng_ust__provider_event_desc___, LTTNG_UST_TRACEPOINT_PROVIDER)[] = {
 #include LTTNG_UST_TRACEPOINT_INCLUDE
 	NULL,	/* Dummy, C99 forbids 0-len array. */
 };
@@ -1192,8 +1200,8 @@ static const struct lttng_ust_event_desc * const LTTNG_UST__TP_COMBINE_TOKENS(lt
 const struct lttng_ust_probe_desc LTTNG_UST__TP_COMBINE_TOKENS(lttng_ust__probe_desc___, LTTNG_UST_TRACEPOINT_PROVIDER) = {
 	.struct_size = sizeof(struct lttng_ust_probe_desc),
 	.provider_name = lttng_ust__tp_stringify(LTTNG_UST_TRACEPOINT_PROVIDER),
-	.event_desc = LTTNG_UST__TP_COMBINE_TOKENS(lttng_ust__event_desc___, LTTNG_UST_TRACEPOINT_PROVIDER),
-	.nr_events = LTTNG_UST__TP_ARRAY_SIZE(LTTNG_UST__TP_COMBINE_TOKENS(lttng_ust__event_desc___, LTTNG_UST_TRACEPOINT_PROVIDER)) - 1,
+	.event_desc = LTTNG_UST__TP_COMBINE_TOKENS(lttng_ust__provider_event_desc___, LTTNG_UST_TRACEPOINT_PROVIDER),
+	.nr_events = LTTNG_UST__TP_ARRAY_SIZE(LTTNG_UST__TP_COMBINE_TOKENS(lttng_ust__provider_event_desc___, LTTNG_UST_TRACEPOINT_PROVIDER)) - 1,
 	.major = LTTNG_UST_PROVIDER_MAJOR,
 	.minor = LTTNG_UST_PROVIDER_MINOR,
 };
