@@ -84,7 +84,16 @@ struct commit_counters_hot {
 #define RB_COMMIT_COUNT_COLD_PADDING	24
 struct commit_counters_cold {
 	union v_atomic cc_sb;		/* Incremented _once_ at sb switch */
-	char padding[RB_COMMIT_COUNT_COLD_PADDING];
+	union {
+		unsigned long end_events_discarded; /*
+						     * Passing events discarded counter
+						     * read upon try_reserve and try_switch
+						     * that fills a subbuffer to check_deliver
+						     * so it can be written into the packet
+						     * header field.
+						     */
+		char padding[RB_COMMIT_COUNT_COLD_PADDING];
+	};
 } __attribute__((aligned(CAA_CACHE_LINE_SIZE)));
 
 /* ring buffer state */
@@ -120,7 +129,7 @@ enum lttng_crash_type {
 
 struct lttng_crash_abi {
 	uint8_t magic[RB_CRASH_DUMP_ABI_MAGIC_LEN];
-	uint64_t mmap_length;	/* Overall lenght of crash record */
+	uint64_t mmap_length;	/* Overall length of crash record */
 	uint16_t endian;	/*
 				 * { 0x12, 0x34 }: big endian
 				 * { 0x34, 0x12 }: little endian
@@ -181,7 +190,7 @@ struct lttng_ust_ring_buffer {
 	int record_disabled;
 	/* End of cache-hot 32 bytes cacheline */
 
-	union v_atomic last_tsc;	/*
+	union v_atomic last_timestamp;	/*
 					 * Last timestamp written in the buffer.
 					 */
 
@@ -251,7 +260,7 @@ struct lttng_ust_ring_buffer_ctx_private {
 						 * prior to record header alignment
 						 * padding.
 						 */
-	uint64_t tsc;				/* time-stamp counter value */
+	uint64_t timestamp;			/* time-stamp counter value */
 	unsigned int rflags;			/* reservation flags */
 	struct lttng_ust_ring_buffer *buf;	/*
 						 * buffer corresponding to processor id
