@@ -18,6 +18,7 @@
 #include <assert.h>
 #include <errno.h>
 #include <fcntl.h>
+#include <inttypes.h>
 
 #include <lttng/ust-ctl.h>
 #include "common/ustcomm.h"
@@ -82,6 +83,7 @@ int ustcomm_connect_unix_sock(const char *pathname, long timeout)
 	strncpy(sun.sun_path, pathname, sizeof(sun.sun_path));
 	sun.sun_path[sizeof(sun.sun_path) - 1] = '\0';
 
+	DBG("Connecting to '%s'", sun.sun_path);
 	ret = connect(fd, (struct sockaddr *) &sun, sizeof(sun));
 	if (ret < 0) {
 		/*
@@ -706,31 +708,31 @@ error:
 	return ret;
 }
 
-ssize_t ustcomm_recv_counter_from_sessiond(int sock,
-		void **_counter_data, uint64_t var_len)
+ssize_t ustcomm_recv_var_len_cmd_from_sessiond(int sock,
+		void **_data, uint32_t var_len)
 {
-	void *counter_data;
+	void *data;
 	ssize_t len;
 
-	if (var_len > LTTNG_UST_ABI_COUNTER_DATA_MAX_LEN) {
+	if (var_len > LTTNG_UST_ABI_CMD_MAX_LEN) {
 		len = -EINVAL;
 		goto error_check;
 	}
 	/* Receive variable length data */
-	counter_data = zmalloc(var_len);
-	if (!counter_data) {
+	data = zmalloc(var_len);
+	if (!data) {
 		len = -ENOMEM;
 		goto error_alloc;
 	}
-	len = ustcomm_recv_unix_sock(sock, counter_data, var_len);
+	len = ustcomm_recv_unix_sock(sock, data, var_len);
 	if (len != var_len) {
 		goto error_recv;
 	}
-	*_counter_data = counter_data;
+	*_data = data;
 	return len;
 
 error_recv:
-	free(counter_data);
+	free(data);
 error_alloc:
 error_check:
 	return len;
@@ -980,7 +982,7 @@ int serialize_dynamic_type(struct lttng_ust_session *session,
 
 	strncpy(uf->name, field_name, LTTNG_UST_ABI_SYM_NAME_LEN);
 	uf->name[LTTNG_UST_ABI_SYM_NAME_LEN - 1] = '\0';
-	uf->type.atype = lttng_ust_ctl_atype_variant;
+	uf->type.atype = lttng_ust_ctl_atype_variant_nestable;
 	uf->type.u.variant_nestable.nr_choices = nr_choices;
 	strncpy(uf->type.u.variant_nestable.tag_name,
 		tag_field_name,
@@ -1367,6 +1369,7 @@ int ustcomm_register_event(int sock,
 	size_t nr_fields,		/* fields */
 	const struct lttng_ust_event_field * const *lttng_fields,
 	const char *model_emf_uri,
+	uint64_t user_token,
 	uint32_t *id)			/* event id (output) */
 {
 	ssize_t len;
@@ -1390,6 +1393,7 @@ int ustcomm_register_event(int sock,
 	strncpy(msg.m.event_name, event_name, LTTNG_UST_ABI_SYM_NAME_LEN);
 	msg.m.event_name[LTTNG_UST_ABI_SYM_NAME_LEN - 1] = '\0';
 	msg.m.loglevel = loglevel;
+	msg.m.user_token = user_token;
 	signature_len = strlen(signature) + 1;
 	msg.m.signature_len = signature_len;
 
@@ -1473,9 +1477,9 @@ int ustcomm_register_event(int sock,
 			return -EINVAL;
 		if (reply.r.ret_code < 0)
 			return reply.r.ret_code;
-		*id = reply.r.event_id;
-		DBG("Sent register event notification for name \"%s\": ret_code %d, event_id %u\n",
-			event_name, reply.r.ret_code, reply.r.event_id);
+		*id = reply.r.id;
+		DBG("Sent register event notification for name \"%s\": ret_code %d, id %" PRIu32 "\n",
+			event_name, reply.r.ret_code, reply.r.id);
 		return 0;
 	default:
 		if (len < 0) {
@@ -1496,6 +1500,115 @@ error_fields:
 	return ret;
 }
 
+#ifdef CONFIG_LTTNG_UST_EXPERIMENTAL_COUNTER
+/*
+ * Returns 0 on success, negative error value on error.
+ * Returns -EPIPE or -ECONNRESET if other end has hung up.
+ */
+int ustcomm_register_key(int sock,
+	int session_objd,		/* session descriptor */
+	int map_objd,			/* map descriptor */
+	uint32_t dimension,
+	const uint64_t *dimension_indexes,
+	const char *key_string,		/* key string (input) */
+	uint64_t user_token,
+	uint64_t *index)		/* (output) */
+{
+	ssize_t len;
+	struct {
+		struct ustcomm_notify_hdr header;
+		struct ustcomm_notify_key_msg m;
+	} msg;
+	struct {
+		struct ustcomm_notify_hdr header;
+		struct ustcomm_notify_key_reply r;
+	} reply;
+	size_t dimension_indexes_len;
+	int ret;
+
+	memset(&msg, 0, sizeof(msg));
+	msg.header.notify_cmd = LTTNG_UST_CTL_NOTIFY_CMD_KEY;
+	msg.m.session_objd = session_objd;
+	msg.m.map_objd = map_objd;
+	msg.m.dimension = dimension;
+	dimension_indexes_len = sizeof(uint64_t) * dimension;
+	msg.m.key_string_len = strlen(key_string) + 1;
+	msg.m.user_token = user_token;
+
+	len = ustcomm_send_unix_sock(sock, &msg, sizeof(msg));
+	if (len > 0 && len != sizeof(msg)) {
+		ret = -EIO;
+		goto error_send;
+	}
+	if (len < 0) {
+		ret = len;
+		goto error_send;
+	}
+
+	/* send dimension_indexes */
+	if (dimension_indexes) {
+		len = ustcomm_send_unix_sock(sock, dimension_indexes, dimension_indexes_len);
+		if (len > 0 && len != dimension_indexes_len) {
+			ret = -EIO;
+			goto error_dimension_indexes;
+		}
+		if (len < 0) {
+			ret = len;
+			goto error_dimension_indexes;
+		}
+	}
+
+	/* send key_string */
+	len = ustcomm_send_unix_sock(sock, key_string, msg.m.key_string_len);
+	if (len > 0 && len != dimension_indexes_len) {
+		ret = -EIO;
+		goto error_dimension_indexes;
+	}
+	if (len < 0) {
+		ret = len;
+		goto error_dimension_indexes;
+	}
+
+	/* receive reply */
+	len = ustcomm_recv_unix_sock(sock, &reply, sizeof(reply));
+	switch (len) {
+	case 0:	/* orderly shutdown */
+		return -EPIPE;
+	case sizeof(reply):
+		if (reply.header.notify_cmd != msg.header.notify_cmd) {
+			ERR("Unexpected result message command "
+				"expected: %u vs received: %u\n",
+				msg.header.notify_cmd, reply.header.notify_cmd);
+			return -EINVAL;
+		}
+		if (reply.r.ret_code > 0)
+			return -EINVAL;
+		if (reply.r.ret_code < 0)
+			return reply.r.ret_code;
+		*index = reply.r.index;
+		DBG("Sent register key notification for key \"%s\": ret_code %d, index %" PRIu64 "\n",
+			key_string, reply.r.ret_code, reply.r.index);
+		return 0;
+	default:
+		if (len < 0) {
+			/* Transport level error */
+			if (errno == EPIPE || errno == ECONNRESET)
+				len = -errno;
+			return len;
+		} else {
+			ERR("incorrect message size: %zd\n", len);
+			return len;
+		}
+	}
+	/* Unreached. */
+
+	/* Error path only. */
+error_dimension_indexes:
+error_send:
+	return ret;
+}
+#endif	/* CONFIG_LTTNG_UST_EXPERIMENTAL_COUNTER */
+
 /*
  * Returns 0 on success, negative error value on error.
  * Returns -EPIPE or -ECONNRESET if other end has hung up.
@@ -1709,7 +1822,7 @@ int ustcomm_register_channel(int sock,
 }
 
 /*
- * Set socket reciving timeout.
+ * Set socket receiving timeout.
  */
 int ustcomm_setsockopt_rcv_timeout(int sock, unsigned int msec)
 {
