@@ -17,6 +17,7 @@
 #include <limits.h>
 #include <unistd.h>
 #include <lttng/ust-abi.h>
+#include <lttng/ust-abi-old.h>
 #include <lttng/ust-error.h>
 #include <lttng/ust-compiler.h>
 #include <lttng/ust-ctl.h>
@@ -90,18 +91,12 @@ struct ustcomm_ust_msg {
 			uint32_t reloc_offset;
 			uint64_t seqnum;
 		} __attribute__((packed)) capture;
-		struct lttng_ust_abi_counter counter;
-		struct lttng_ust_abi_counter_global counter_global;
-		struct lttng_ust_abi_counter_cpu counter_cpu;
-		/*
-		 * For lttng_ust_abi_EVENT_NOTIFIER_CREATE, a struct
-		 * lttng_ust_abi_event_notifier implicitly follows struct
-		 * ustcomm_ust_msg.
-		 */
+		struct lttng_ust_abi_old_counter counter_old;
+		struct lttng_ust_abi_old_counter_channel counter_channel_old;
+		struct lttng_ust_abi_old_counter_cpu counter_cpu_old;
 		struct {
-			/* Length of struct lttng_ust_abi_event_notifier */
-			uint32_t len;
-		} event_notifier;
+			uint32_t cmd_len;
+		} __attribute__((packed)) var_len_cmd;
 		char padding[USTCOMM_MSG_PADDING2];
 	} u;
 } __attribute__((packed));
@@ -135,7 +130,7 @@ struct ustcomm_notify_hdr {
 	uint32_t notify_cmd;
 } __attribute__((packed));
 
-#define USTCOMM_NOTIFY_EVENT_MSG_PADDING	32
+#define USTCOMM_NOTIFY_EVENT_MSG_PADDING	24
 struct ustcomm_notify_event_msg {
 	uint32_t session_objd;
 	uint32_t channel_objd;
@@ -144,6 +139,7 @@ struct ustcomm_notify_event_msg {
 	uint32_t signature_len;
 	uint32_t fields_len;
 	uint32_t model_emf_uri_len;
+	uint64_t user_token;
 	char padding[USTCOMM_NOTIFY_EVENT_MSG_PADDING];
 	/* followed by signature, fields, and model_emf_uri */
 } __attribute__((packed));
@@ -151,10 +147,28 @@ struct ustcomm_notify_event_msg {
 #define USTCOMM_NOTIFY_EVENT_REPLY_PADDING	32
 struct ustcomm_notify_event_reply {
 	int32_t ret_code;	/* 0: ok, negative: error code */
-	uint32_t event_id;
+	uint32_t id;	/* 32-bit event id. */
 	char padding[USTCOMM_NOTIFY_EVENT_REPLY_PADDING];
 } __attribute__((packed));
 
+#define USTCOMM_NOTIFY_KEY_MSG_PADDING	24
+struct ustcomm_notify_key_msg {
+	uint32_t session_objd;
+	uint32_t map_objd;
+	uint32_t dimension;
+	uint32_t key_string_len;
+	uint64_t user_token;
+	char padding[USTCOMM_NOTIFY_KEY_MSG_PADDING];
+	/* followed by dimension_indexes (array of @dimension uint64_t items) and key_string. */
+} __attribute__((packed));
+
+#define USTCOMM_NOTIFY_KEY_REPLY_PADDING	32
+struct ustcomm_notify_key_reply {
+	int32_t ret_code;	/* 0: ok, negative: error code */
+	uint64_t index;		/* 64-bit key index. */
+	char padding[USTCOMM_NOTIFY_KEY_REPLY_PADDING];
+} __attribute__((packed));
+
 #define USTCOMM_NOTIFY_ENUM_MSG_PADDING		32
 struct ustcomm_notify_enum_msg {
 	uint32_t session_objd;
@@ -164,11 +178,11 @@ struct ustcomm_notify_enum_msg {
 	/* followed by enum entries */
 } __attribute__((packed));
 
-#define USTCOMM_NOTIFY_EVENT_REPLY_PADDING	32
+#define USTCOMM_NOTIFY_ENUM_REPLY_PADDING	32
 struct ustcomm_notify_enum_reply {
 	int32_t ret_code;	/* 0: ok, negative: error code */
 	uint64_t enum_id;
-	char padding[USTCOMM_NOTIFY_EVENT_REPLY_PADDING];
+	char padding[USTCOMM_NOTIFY_ENUM_REPLY_PADDING];
 } __attribute__((packed));
 
 #define USTCOMM_NOTIFY_CHANNEL_MSG_PADDING	32
@@ -255,8 +269,8 @@ ssize_t ustcomm_recv_event_notifier_notif_fd_from_sessiond(int sock,
 		int *event_notifier_notif_fd)
 	__attribute__((visibility("hidden")));
 
-ssize_t ustcomm_recv_counter_from_sessiond(int sock,
-		void **counter_data, uint64_t len)
+ssize_t ustcomm_recv_var_len_cmd_from_sessiond(int sock,
+		void **data, uint32_t len)
 	__attribute__((visibility("hidden")));
 
 int ustcomm_recv_counter_shm_from_sessiond(int sock,
@@ -292,9 +306,26 @@ int ustcomm_register_event(int sock,
 	size_t nr_fields,		/* fields */
 	const struct lttng_ust_event_field * const *fields,
 	const char *model_emf_uri,
-	uint32_t *id)			/* event id (output) */
+	uint64_t user_token,
+	uint32_t *id)			/* (output) */
 	__attribute__((visibility("hidden")));
 
+#ifdef CONFIG_LTTNG_UST_EXPERIMENTAL_COUNTER
+/*
+ * Returns 0 on success, negative error value on error.
+ * Returns -EPIPE or -ECONNRESET if other end has hung up.
+ */
+int ustcomm_register_key(int sock,
+	int session_objd,		/* session descriptor */
+	int map_objd,			/* map descriptor */
+	uint32_t dimension,
+	const uint64_t *dimension_indexes,
+	const char *key_string,		/* key string (input) */
+	uint64_t user_token,
+	uint64_t *index)		/* (output) */
+	__attribute__((visibility("hidden")));
+#endif	/* CONFIG_LTTNG_UST_EXPERIMENTAL_COUNTER */
+
 /*
  * Returns 0 on success, negative error value on error.
  * Returns -EPIPE or -ECONNRESET if other end has hung up.
