@@ -154,7 +154,9 @@ error_find_context:
 /*
  * Force a read (imply TLS allocation for dlopen) of TLS variables.
  */
-void lttng_uts_ns_alloc_tls(void)
+void lttng_ust_uts_ns_init_thread(int flags)
 {
 	__asm__ __volatile__ ("" : : "m" (URCU_TLS(cached_uts_ns)));
+	if (flags & LTTNG_UST_INIT_THREAD_CONTEXT_CACHE)
+		(void)get_uts_ns();
 }
