@@ -38,6 +38,7 @@
 
 #include "common/ust-fd.h"
 #include "common/logging.h"
+#include "common/align.h"
 
 #include "common/ringbuffer/frontend_types.h"
 #include "common/ringbuffer/frontend.h"
@@ -275,6 +276,7 @@ static const struct lttng_ust_abi_objd_ops lttng_ops;
 static const struct lttng_ust_abi_objd_ops lttng_event_notifier_group_ops;
 static const struct lttng_ust_abi_objd_ops lttng_session_ops;
 static const struct lttng_ust_abi_objd_ops lttng_channel_ops;
+static const struct lttng_ust_abi_objd_ops lttng_counter_ops;
 static const struct lttng_ust_abi_objd_ops lttng_event_enabler_ops;
 static const struct lttng_ust_abi_objd_ops lttng_event_notifier_enabler_ops;
 static const struct lttng_ust_abi_objd_ops lttng_tracepoint_list_ops;
@@ -469,6 +471,8 @@ int lttng_abi_map_channel(int session_objd,
 	switch (type) {
 	case LTTNG_UST_ABI_CHAN_PER_CPU:
 		break;
+	case LTTNG_UST_ABI_CHAN_PER_CHANNEL:
+		break;
 	default:
 		ret = -EINVAL;
 		goto invalid;
@@ -533,6 +537,27 @@ int lttng_abi_map_channel(int session_objd,
 		}
 		chan_name = "channel";
 		break;
+	case LTTNG_UST_ABI_CHAN_PER_CHANNEL:
+		if (config->output == RING_BUFFER_MMAP) {
+			if (config->mode == RING_BUFFER_OVERWRITE) {
+				if (config->wakeup == RING_BUFFER_WAKEUP_BY_WRITER) {
+					transport_name = "relay-overwrite-channel-mmap";
+				} else {
+					transport_name = "relay-overwrite-channel-rt-mmap";
+				}
+			} else {
+				if (config->wakeup == RING_BUFFER_WAKEUP_BY_WRITER) {
+					transport_name = "relay-discard-channel-mmap";
+				} else {
+					transport_name = "relay-discard-channel-rt-mmap";
+				}
+			}
+		} else {
+			ret = -EINVAL;
+			goto notransport;
+		}
+		chan_name = "channel";
+		break;
 	default:
 		ret = -EINVAL;
 		goto notransport;
@@ -599,6 +624,149 @@ invalid:
 	return ret;
 }
 
+static
+bool check_zero(const char *p, size_t len)
+{
+	size_t i;
+
+	for (i = 0; i < len; i++) {
+		if (p[i] != 0)
+			return false;
+	}
+	return true;
+}
+
+static
+int copy_abi_struct(void *dst_struct, size_t dst_struct_len,
+		const void *src_struct, size_t src_struct_len)
+{
+	if (dst_struct_len >= src_struct_len) {
+		memcpy(dst_struct, src_struct, src_struct_len);
+		if (dst_struct_len > src_struct_len)
+			memset(dst_struct + src_struct_len, 0, dst_struct_len - src_struct_len);
+	} else {	/* dst_struct_len < src_struct_len */
+		/* Validate zero-padding. */
+		if (!check_zero(src_struct + dst_struct_len, src_struct_len - dst_struct_len))
+			return -E2BIG;
+		memcpy(dst_struct, src_struct, dst_struct_len);
+	}
+	return 0;
+}
+
+static
+long lttng_session_create_counter(
+		int session_objd,
+		const struct lttng_ust_abi_counter_conf *abi_counter_conf,
+		union lttng_ust_abi_args *uargs,
+		void *owner)
+{
+	struct lttng_ust_session *session = objd_private(session_objd);
+	int counter_objd, ret;
+	const char *counter_transport_name;
+	struct lttng_ust_channel_counter *counter = NULL;
+	struct lttng_counter_dimension dimensions[1] = {};
+	size_t number_dimensions = 1;
+	struct lttng_ust_abi_counter_conf counter_conf;
+	uint32_t min_expected_len = lttng_ust_offsetofend(struct lttng_ust_abi_counter_conf, elem_len);
+	const struct lttng_ust_abi_counter_dimension *abi_dimension;
+	struct lttng_ust_abi_counter_dimension dimension;
+
+	if (uargs->counter.len < min_expected_len) {
+		ERR("LTTng: Map: Counter configuration of wrong size.");
+		return -EINVAL;
+	}
+	if (abi_counter_conf->len > uargs->counter.len || abi_counter_conf->len < lttng_ust_offsetofend(struct lttng_ust_abi_counter_conf, elem_len)) {
+		return -EINVAL;
+	}
+	ret = copy_abi_struct(&counter_conf, sizeof(counter_conf), abi_counter_conf, abi_counter_conf->len);
+	if (ret) {
+		ERR("Unexpected counter configuration structure content");
+		return ret;
+	}
+	if (counter_conf.number_dimensions != 1) {
+		ERR("LTTng: Map: Unsupprted number of dimensions %u.", counter_conf.number_dimensions);
+		return -EINVAL;
+	}
+	if (counter_conf.elem_len < lttng_ust_offsetofend(struct lttng_ust_abi_counter_dimension, overflow_index)) {
+		ERR("Unexpected dimension array element length %u.", counter_conf.elem_len);
+		return -EINVAL;
+	}
+	if (counter_conf.len + counter_conf.elem_len > uargs->counter.len) {
+		return -EINVAL;
+	}
+	abi_dimension = (const struct lttng_ust_abi_counter_dimension *)(((char *)abi_counter_conf) + counter_conf.len);
+	ret = copy_abi_struct(&dimension, sizeof(dimension), abi_dimension, counter_conf.elem_len);
+	if (ret) {
+		ERR("Unexpected dimension structure content");
+		return ret;
+	}
+	if (counter_conf.arithmetic != LTTNG_UST_ABI_COUNTER_ARITHMETIC_MODULAR) {
+		ERR("LTTng: Map: Counter of the wrong type.");
+		return -EINVAL;
+	}
+	if (counter_conf.global_sum_step) {
+		/* Unsupported. */
+		return -EINVAL;
+	}
+	switch (counter_conf.bitness) {
+	case LTTNG_UST_ABI_COUNTER_BITNESS_64:
+		counter_transport_name = "counter-per-cpu-64-modular";
+		break;
+	case LTTNG_UST_ABI_COUNTER_BITNESS_32:
+		counter_transport_name = "counter-per-cpu-32-modular";
+		break;
+	default:
+		return -EINVAL;
+	}
+
+	dimensions[0].size = dimension.size;
+	dimensions[0].underflow_index = dimension.underflow_index;
+	dimensions[0].overflow_index = dimension.overflow_index;
+	dimensions[0].has_underflow = dimension.flags & LTTNG_UST_ABI_COUNTER_DIMENSION_FLAG_UNDERFLOW;
+	dimensions[0].has_overflow = dimension.flags & LTTNG_UST_ABI_COUNTER_DIMENSION_FLAG_OVERFLOW;
+	switch (dimension.key_type) {
+	case LTTNG_UST_ABI_KEY_TYPE_TOKENS:
+		dimensions[0].key_type = LTTNG_KEY_TYPE_TOKENS;
+		break;
+	case LTTNG_UST_ABI_KEY_TYPE_INTEGER:	/* Fall-through */
+	default:
+		return -EINVAL;
+	}
+
+	counter_objd = objd_alloc(NULL, &lttng_counter_ops, owner, "counter");
+	if (counter_objd < 0) {
+		ret = counter_objd;
+		goto objd_error;
+	}
+
+	counter = lttng_ust_counter_create(counter_transport_name,
+			number_dimensions, dimensions,
+			0, counter_conf.flags & LTTNG_UST_ABI_COUNTER_CONF_FLAG_COALESCE_HITS);
+	if (!counter) {
+		ret = -EINVAL;
+		goto counter_error;
+	}
+	counter->parent->session = session;
+	cds_list_add(&counter->priv->node, &session->priv->counters_head);
+	objd_set_private(counter_objd, counter);
+	counter->priv->parent.objd = counter_objd;
+	counter->priv->parent.tstate = 1;
+	counter->parent->enabled = 1;
+	/* The channel created holds a reference on the session */
+	objd_ref(session_objd);
+	return counter_objd;
+
+counter_error:
+	{
+		int err;
+
+		err = lttng_ust_abi_objd_unref(counter_objd, 1);
+		assert(!err);
+	}
+objd_error:
+	return ret;
+}
+
 /**
  *	lttng_session_cmd - lttng session object command
  *
@@ -638,10 +806,9 @@ long lttng_session_cmd(int objd, unsigned int cmd, unsigned long arg,
 	case LTTNG_UST_ABI_SESSION_STATEDUMP:
 		return lttng_session_statedump(session);
 	case LTTNG_UST_ABI_COUNTER:
-	case LTTNG_UST_ABI_COUNTER_GLOBAL:
-	case LTTNG_UST_ABI_COUNTER_CPU:
-		/* Not implemented yet. */
-		return -EINVAL;
+		return lttng_session_create_counter(objd,
+				(struct lttng_ust_abi_counter_conf *)arg,
+				uargs, owner);
 	default:
 		return -EINVAL;
 	}
@@ -722,20 +889,20 @@ long lttng_event_notifier_enabler_cmd(int objd, unsigned int cmd, unsigned long
 	struct lttng_event_notifier_enabler *event_notifier_enabler = objd_private(objd);
 	switch (cmd) {
 	case LTTNG_UST_ABI_FILTER:
-		return lttng_event_notifier_enabler_attach_filter_bytecode(
-			event_notifier_enabler,
+		return lttng_event_enabler_attach_filter_bytecode(
+			&event_notifier_enabler->parent,
 			(struct lttng_ust_bytecode_node **) arg);
 	case LTTNG_UST_ABI_EXCLUSION:
-		return lttng_event_notifier_enabler_attach_exclusion(event_notifier_enabler,
+		return lttng_event_enabler_attach_exclusion(&event_notifier_enabler->parent,
 			(struct lttng_ust_excluder_node **) arg);
 	case LTTNG_UST_ABI_CAPTURE:
 		return lttng_event_notifier_enabler_attach_capture_bytecode(
 			event_notifier_enabler,
 			(struct lttng_ust_bytecode_node **) arg);
 	case LTTNG_UST_ABI_ENABLE:
-		return lttng_event_notifier_enabler_enable(event_notifier_enabler);
+		return lttng_event_enabler_enable(&event_notifier_enabler->parent);
 	case LTTNG_UST_ABI_DISABLE:
-		return lttng_event_notifier_enabler_disable(event_notifier_enabler);
+		return lttng_event_enabler_disable(&event_notifier_enabler->parent);
 	default:
 		return -EINVAL;
 	}
@@ -751,7 +918,7 @@ long lttng_event_notifier_enabler_cmd(int objd, unsigned int cmd, unsigned long
  *	@owner: objd owner
  *
  *	This descriptor implements lttng commands:
- *      LTTNG_UST_ABI_COUNTER_GLOBAL
+ *      LTTNG_UST_ABI_COUNTER_CHANNEL
  *        Return negative error code on error, 0 on success.
  *      LTTNG_UST_ABI_COUNTER_CPU
  *        Return negative error code on error, 0 on success.
@@ -761,19 +928,27 @@ long lttng_event_notifier_group_error_counter_cmd(int objd, unsigned int cmd, un
 	union lttng_ust_abi_args *uargs, void *owner __attribute__((unused)))
 {
 	int ret;
-	struct lttng_counter *counter = objd_private(objd);
+	struct lttng_ust_channel_counter *counter = objd_private(objd);
 
 	switch (cmd) {
-	case LTTNG_UST_ABI_COUNTER_GLOBAL:
+	case LTTNG_UST_ABI_COUNTER_CHANNEL:
 		ret = -EINVAL;	/* Unimplemented. */
 		break;
 	case LTTNG_UST_ABI_COUNTER_CPU:
 	{
-		struct lttng_ust_abi_counter_cpu *counter_cpu =
-			(struct lttng_ust_abi_counter_cpu *)arg;
+		struct lttng_ust_abi_counter_cpu *abi_counter_cpu =
+			(struct lttng_ust_abi_counter_cpu *) arg;
+		struct lttng_ust_abi_counter_cpu counter_cpu;
 
-		ret = lttng_counter_set_cpu_shm(counter->counter,
-			counter_cpu->cpu_nr, uargs->counter_shm.shm_fd);
+		if (abi_counter_cpu->len < lttng_ust_offsetofend(struct lttng_ust_abi_counter_cpu, cpu_nr)) {
+			return -EINVAL;
+		}
+		ret = copy_abi_struct(&counter_cpu, sizeof(counter_cpu),
+				abi_counter_cpu, abi_counter_cpu->len);
+		if (ret)
+			return ret;
+		ret = lttng_counter_set_cpu_shm(counter->priv->counter,
+			counter_cpu.cpu_nr, uargs->counter_shm.shm_fd);
 		if (!ret) {
 			/* Take ownership of the shm_fd. */
 			uargs->counter_shm.shm_fd = -1;
@@ -792,10 +967,10 @@ int lttng_release_event_notifier_group_error_counter(int objd)
 	__attribute__((visibility("hidden")));
 int lttng_release_event_notifier_group_error_counter(int objd)
 {
-	struct lttng_counter *counter = objd_private(objd);
+	struct lttng_ust_channel_counter *counter = objd_private(objd);
 
 	if (counter) {
-		return lttng_ust_abi_objd_unref(counter->event_notifier_group->objd, 0);
+		return lttng_ust_abi_objd_unref(counter->priv->event_notifier_group->objd, 0);
 	} else {
 		return -EINVAL;
 	}
@@ -807,27 +982,64 @@ static const struct lttng_ust_abi_objd_ops lttng_event_notifier_group_error_coun
 };
 
 static
-int lttng_ust_event_notifier_group_create_error_counter(int event_notifier_group_objd, void *owner,
-		struct lttng_ust_abi_counter_conf *error_counter_conf)
+int lttng_ust_event_notifier_group_create_error_counter(int event_notifier_group_objd,
+		struct lttng_ust_abi_counter_conf *abi_counter_conf,
+		union lttng_ust_abi_args *uargs,
+		void *owner)
 {
 	const char *counter_transport_name;
 	struct lttng_event_notifier_group *event_notifier_group =
 		objd_private(event_notifier_group_objd);
-	struct lttng_counter *counter;
+	struct lttng_ust_channel_counter *counter;
 	int counter_objd, ret;
-	struct lttng_counter_dimension dimensions[1];
 	size_t counter_len;
+	struct lttng_counter_dimension dimensions[1] = {};
+	struct lttng_ust_abi_counter_conf counter_conf;
+	uint32_t min_expected_len = lttng_ust_offsetofend(struct lttng_ust_abi_counter_conf, elem_len);
+	const struct lttng_ust_abi_counter_dimension *abi_dimension;
+	struct lttng_ust_abi_counter_dimension dimension;
 
 	if (event_notifier_group->error_counter)
 		return -EBUSY;
 
-	if (error_counter_conf->arithmetic != LTTNG_UST_ABI_COUNTER_ARITHMETIC_MODULAR)
+	if (uargs->counter.len < min_expected_len) {
+		ERR("LTTng: Counter configuration of wrong size.");
 		return -EINVAL;
-
-	if (error_counter_conf->number_dimensions != 1)
+	}
+	if (abi_counter_conf->len > uargs->counter.len || abi_counter_conf->len < lttng_ust_offsetofend(struct lttng_ust_abi_counter_conf, elem_len)) {
 		return -EINVAL;
-
-	switch (error_counter_conf->bitness) {
+	}
+	ret = copy_abi_struct(&counter_conf, sizeof(counter_conf), abi_counter_conf, abi_counter_conf->len);
+	if (ret) {
+		ERR("Unexpected counter configuration structure content");
+		return ret;
+	}
+	if (counter_conf.number_dimensions != 1) {
+		ERR("LTTng: Map: Unsupprted number of dimensions %u.", counter_conf.number_dimensions);
+		return -EINVAL;
+	}
+	if (counter_conf.elem_len < lttng_ust_offsetofend(struct lttng_ust_abi_counter_dimension, overflow_index)) {
+		ERR("Unexpected dimension array element length %u.", counter_conf.elem_len);
+		return -EINVAL;
+	}
+	if (counter_conf.len + counter_conf.elem_len > uargs->counter.len) {
+		return -EINVAL;
+	}
+	abi_dimension = (const struct lttng_ust_abi_counter_dimension *)(((char *)abi_counter_conf) + counter_conf.len);
+	ret = copy_abi_struct(&dimension, sizeof(dimension), abi_dimension, counter_conf.elem_len);
+	if (ret) {
+		ERR("Unexpected dimension structure content");
+		return ret;
+	}
+	if (counter_conf.arithmetic != LTTNG_UST_ABI_COUNTER_ARITHMETIC_MODULAR) {
+		ERR("LTTng: Counter of the wrong type.");
+		return -EINVAL;
+	}
+	if (counter_conf.global_sum_step) {
+		/* Unsupported. */
+		return -EINVAL;
+	}
+	switch (counter_conf.bitness) {
 	case LTTNG_UST_ABI_COUNTER_BITNESS_64:
 		counter_transport_name = "counter-per-cpu-64-modular";
 		break;
@@ -838,6 +1050,13 @@ int lttng_ust_event_notifier_group_create_error_counter(int event_notifier_group
 		return -EINVAL;
 	}
 
+	counter_len = dimension.size;
+	dimensions[0].size = counter_len;
+	dimensions[0].underflow_index = dimension.underflow_index;
+	dimensions[0].overflow_index = dimension.overflow_index;
+	dimensions[0].has_underflow = dimension.flags & LTTNG_UST_ABI_COUNTER_DIMENSION_FLAG_UNDERFLOW;
+	dimensions[0].has_overflow = dimension.flags & LTTNG_UST_ABI_COUNTER_DIMENSION_FLAG_OVERFLOW;
+
 	counter_objd = objd_alloc(NULL, &lttng_event_notifier_group_error_counter_ops, owner,
 		"event_notifier group error counter");
 	if (counter_objd < 0) {
@@ -845,14 +1064,7 @@ int lttng_ust_event_notifier_group_create_error_counter(int event_notifier_group
 		goto objd_error;
 	}
 
-	counter_len = error_counter_conf->dimensions[0].size;
-	dimensions[0].size = counter_len;
-	dimensions[0].underflow_index = 0;
-	dimensions[0].overflow_index = 0;
-	dimensions[0].has_underflow = 0;
-	dimensions[0].has_overflow = 0;
-
-	counter = lttng_ust_counter_create(counter_transport_name, 1, dimensions);
+	counter = lttng_ust_counter_create(counter_transport_name, 1, dimensions, 0, false);
 	if (!counter) {
 		ret = -EINVAL;
 		goto create_error;
@@ -869,8 +1081,8 @@ int lttng_ust_event_notifier_group_create_error_counter(int event_notifier_group
 	cmm_smp_mb();
 	CMM_STORE_SHARED(event_notifier_group->error_counter, counter);
 
-	counter->objd = counter_objd;
-	counter->event_notifier_group = event_notifier_group;	/* owner */
+	counter->priv->parent.objd = counter_objd;
+	counter->priv->event_notifier_group = event_notifier_group;	/* owner */
 
 	objd_set_private(counter_objd, counter);
 	/* The error counter holds a reference on the event_notifier group. */
@@ -893,31 +1105,40 @@ static
 long lttng_event_notifier_group_cmd(int objd, unsigned int cmd, unsigned long arg,
 		union lttng_ust_abi_args *uargs, void *owner)
 {
+	int ret;
+
 	switch (cmd) {
 	case LTTNG_UST_ABI_EVENT_NOTIFIER_CREATE:
 	{
-		struct lttng_ust_abi_event_notifier *event_notifier_param =
+		struct lttng_ust_abi_event_notifier *abi_event_notifier =
 			(struct lttng_ust_abi_event_notifier *) arg;
-		if (strutils_is_star_glob_pattern(event_notifier_param->event.name)) {
+		struct lttng_ust_abi_event_notifier event_notifier = {};
+
+		if (uargs->event_notifier.len < lttng_ust_offsetofend(struct lttng_ust_abi_event_notifier, error_counter_index))
+			return -EINVAL;
+		ret = copy_abi_struct(&event_notifier, sizeof(event_notifier),
+				abi_event_notifier, uargs->event_notifier.len);
+		if (ret)
+			return ret;
+		event_notifier.event.name[LTTNG_UST_ABI_SYM_NAME_LEN - 1] = '\0';
+		if (strutils_is_star_glob_pattern(event_notifier.event.name)) {
 			/*
 			 * If the event name is a star globbing pattern,
 			 * we create the special star globbing enabler.
 			 */
 			return lttng_ust_event_notifier_enabler_create(objd,
-					owner, event_notifier_param,
+					owner, &event_notifier,
 					LTTNG_ENABLER_FORMAT_STAR_GLOB);
 		} else {
 			return lttng_ust_event_notifier_enabler_create(objd,
-					owner, event_notifier_param,
+					owner, &event_notifier,
 					LTTNG_ENABLER_FORMAT_EVENT);
 		}
 	}
 	case LTTNG_UST_ABI_COUNTER:
 	{
-		struct lttng_ust_abi_counter_conf *counter_conf =
-			(struct lttng_ust_abi_counter_conf *) uargs->counter.counter_data;
 		return lttng_ust_event_notifier_group_create_error_counter(
-				objd, owner, counter_conf);
+				objd, (struct lttng_ust_abi_counter_conf *) arg, uargs, owner);
 	}
 	default:
 		return -EINVAL;
@@ -1142,18 +1363,18 @@ error_add_stream:
 }
 
 static
-int lttng_abi_create_event_enabler(int channel_objd,
-			   struct lttng_ust_abi_event *event_param,
-			   void *owner,
-			   enum lttng_enabler_format_type format_type)
+int lttng_abi_create_event_recorder_enabler(int channel_objd,
+			struct lttng_ust_channel_buffer *channel,
+			struct lttng_ust_abi_event *event_param,
+			void *owner,
+			enum lttng_enabler_format_type format_type)
 {
-	struct lttng_ust_channel_buffer *channel = objd_private(channel_objd);
-	struct lttng_event_enabler *enabler;
+	struct lttng_event_recorder_enabler *enabler;
 	int event_objd, ret;
 
 	event_param->name[LTTNG_UST_ABI_SYM_NAME_LEN - 1] = '\0';
 	event_objd = objd_alloc(NULL, &lttng_event_enabler_ops, owner,
-		"event enabler");
+		"event recorder enabler");
 	if (event_objd < 0) {
 		ret = event_objd;
 		goto objd_error;
@@ -1162,12 +1383,13 @@ int lttng_abi_create_event_enabler(int channel_objd,
 	 * We tolerate no failure path after event creation. It will stay
 	 * invariant for the rest of the session.
 	 */
-	enabler = lttng_event_enabler_create(format_type, event_param, channel);
+	enabler = lttng_event_recorder_enabler_create(format_type, event_param,
+			channel);
 	if (!enabler) {
 		ret = -ENOMEM;
 		goto event_error;
 	}
-	objd_set_private(event_objd, enabler);
+	objd_set_private(event_objd, &enabler->parent);
 	/* The event holds a reference on the channel */
 	objd_ref(channel_objd);
 	return event_objd;
@@ -1183,6 +1405,199 @@ objd_error:
 	return ret;
 }
 
+#ifdef CONFIG_LTTNG_UST_EXPERIMENTAL_COUNTER
+static
+int copy_counter_key_dimension_tokens(const struct lttng_ust_abi_counter_key_dimension_tokens *abi_dim_tokens,
+		const char *addr, size_t *offset, size_t arg_len, struct lttng_counter_key_dimension *internal_dim)
+{
+	struct lttng_ust_abi_counter_key_dimension_tokens dim_tokens;
+	size_t nr_key_tokens, j;
+	int ret;
+
+	if (abi_dim_tokens->parent.len < sizeof(struct lttng_ust_abi_counter_key_dimension_tokens))
+		return -EINVAL;
+	ret = copy_abi_struct(&dim_tokens, sizeof(dim_tokens), abi_dim_tokens, abi_dim_tokens->parent.len);
+	if (ret)
+		return ret;
+	nr_key_tokens = dim_tokens.nr_key_tokens;
+	if (!nr_key_tokens || nr_key_tokens > LTTNG_NR_KEY_TOKEN)
+		return -EINVAL;
+	internal_dim->key_type = LTTNG_KEY_TYPE_TOKENS;
+	internal_dim->u.tokens.nr_key_tokens = nr_key_tokens;
+	*offset += sizeof(struct lttng_ust_abi_counter_key_dimension_tokens);
+	for (j = 0; j < nr_key_tokens; j++) {
+		struct lttng_key_token *internal_token = &internal_dim->u.tokens.key_tokens[j];
+		const struct lttng_ust_abi_key_token *abi_token;
+
+		if (*offset + sizeof(struct lttng_ust_abi_key_token) > arg_len)
+			return -EINVAL;
+		abi_token = (const struct lttng_ust_abi_key_token *)(addr + *offset);
+		if (abi_token->len < sizeof(struct lttng_ust_abi_key_token))
+			return -EINVAL;
+		if (*offset + abi_token->len > arg_len)
+			return -EINVAL;
+		switch (abi_token->type) {
+		case LTTNG_UST_ABI_KEY_TOKEN_STRING:
+		{
+			const struct lttng_ust_abi_key_token_string *abi_key_string;
+			struct lttng_ust_abi_key_token_string token_string;
+
+			if (abi_token->len < sizeof(struct lttng_ust_abi_key_token_string))
+				return -EINVAL;
+			abi_key_string = (const struct lttng_ust_abi_key_token_string *)(addr + *offset);
+			ret = copy_abi_struct(&token_string, sizeof(token_string), abi_key_string, abi_key_string->parent.len);
+			if (ret)
+				return ret;
+			*offset += abi_key_string->parent.len;
+			internal_token->type = LTTNG_KEY_TOKEN_STRING;
+			if (!abi_key_string->string_len || abi_key_string->string_len > LTTNG_KEY_TOKEN_STRING_LEN_MAX)
+				return -EINVAL;
+			*offset += abi_key_string->string_len;
+			if (*offset > arg_len)
+				return -EINVAL;
+			if (abi_key_string->str[abi_key_string->string_len - 1] != '\0' ||
+					strlen(abi_key_string->str) + 1 != abi_key_string->string_len)
+				return -EINVAL;
+			memcpy(internal_token->arg.string, abi_key_string->str, abi_key_string->string_len);
+			break;
+		}
+		case LTTNG_UST_ABI_KEY_TOKEN_EVENT_NAME:
+			internal_token->type = LTTNG_KEY_TOKEN_EVENT_NAME;
+			*offset += abi_token->len;
+			break;
+		case LTTNG_UST_ABI_KEY_TOKEN_PROVIDER_NAME:
+			internal_token->type = LTTNG_KEY_TOKEN_PROVIDER_NAME;
+			*offset += abi_token->len;
+			break;
+		default:
+			return -EINVAL;
+		}
+	}
+	return 0;
+}
+
+static
+int copy_counter_key(struct lttng_counter_key *internal_key,
+		     unsigned long arg, size_t action_fields_len, size_t arg_len,
+		     const struct lttng_ust_abi_counter_event *counter_event)
+{
+	size_t i, nr_dimensions, offset = 0;
+	const char *addr = (const char *)arg;
+	int ret;
+
+	nr_dimensions = counter_event->number_key_dimensions;
+	if (nr_dimensions != 1)
+		return -EINVAL;
+	internal_key->nr_dimensions = nr_dimensions;
+	offset += counter_event->len + action_fields_len;
+	for (i = 0; i < nr_dimensions; i++) {
+		struct lttng_counter_key_dimension *internal_dim = &internal_key->key_dimensions[i];
+		const struct lttng_ust_abi_counter_key_dimension *abi_dim;
+
+		abi_dim = (const struct lttng_ust_abi_counter_key_dimension *)(addr + offset);
+		if (offset + abi_dim->len > arg_len || abi_dim->len < lttng_ust_offsetofend(struct lttng_ust_abi_counter_key_dimension, key_type))
+			return -EINVAL;
+		switch (abi_dim->key_type) {
+		case LTTNG_UST_ABI_KEY_TYPE_TOKENS:
+		{
+			struct lttng_ust_abi_counter_key_dimension_tokens *dim_tokens =
+				caa_container_of(abi_dim, struct lttng_ust_abi_counter_key_dimension_tokens, parent);
+			ret = copy_counter_key_dimension_tokens(dim_tokens, addr, &offset, arg_len,
+					internal_dim);
+			if (ret)
+				return ret;
+			break;
+		}
+		default:
+			return -EINVAL;
+		}
+	}
+	return 0;
+}
+
+static
+int lttng_abi_create_event_counter_enabler(int channel_objd,
+			struct lttng_ust_channel_counter *channel,
+			unsigned long arg, size_t arg_len, void *owner)
+{
+	struct lttng_ust_abi_counter_event *abi_counter_event = (struct lttng_ust_abi_counter_event *)arg;
+	struct lttng_ust_abi_counter_event counter_event = {};
+	struct lttng_counter_key counter_key = {};
+	struct lttng_event_counter_enabler *enabler;
+	enum lttng_enabler_format_type format_type;
+	size_t action_fields_len = 0;
+	int event_objd, ret;
+	size_t i;
+
+	if (arg_len < lttng_ust_offsetofend(struct lttng_ust_abi_counter_event, number_key_dimensions)) {
+		return -EINVAL;
+	}
+	if (arg_len < abi_counter_event->len ||
+			abi_counter_event->len < lttng_ust_offsetofend(struct lttng_ust_abi_counter_event, number_key_dimensions)) {
+		return -EINVAL;
+	}
+	ret = copy_abi_struct(&counter_event, sizeof(counter_event),
+			abi_counter_event, abi_counter_event->len);
+	if (ret) {
+		return ret;
+	}
+	switch (counter_event.action) {
+	case LTTNG_UST_ABI_COUNTER_ACTION_INCREMENT:
+		/* No additional fields specific to this action. */
+		break;
+	default:
+		return -EINVAL;
+	}
+	counter_event.event.name[LTTNG_UST_ABI_SYM_NAME_LEN - 1] = '\0';
+	if (strutils_is_star_glob_pattern(counter_event.event.name)) {
+		format_type = LTTNG_ENABLER_FORMAT_STAR_GLOB;
+	} else {
+		format_type = LTTNG_ENABLER_FORMAT_EVENT;
+	}
+	ret = copy_counter_key(&counter_key, arg, action_fields_len, arg_len, &counter_event);
+	if (ret) {
+		return ret;
+	}
+	/*
+	 * Validate that each dimension counter key type match the map
+	 * key type.
+	 */
+	for (i = 0; i < counter_key.nr_dimensions; i++) {
+		if (channel->priv->dimension_key_types[i] != counter_key.key_dimensions[i].key_type)
+			return -EINVAL;
+	}
+	event_objd = objd_alloc(NULL, &lttng_event_enabler_ops, owner,
+		"event enabler");
+	if (event_objd < 0) {
+		ret = event_objd;
+		goto objd_error;
+	}
+	/*
+	 * We tolerate no failure path after event creation. It will stay
+	 * invariant for the rest of the session.
+	 */
+	enabler = lttng_event_counter_enabler_create(format_type, &counter_event, &counter_key, channel);
+	if (!enabler) {
+		ret = -ENOMEM;
+		goto event_error;
+	}
+	objd_set_private(event_objd, &enabler->parent);
+	/* The event holds a reference on the channel */
+	objd_ref(channel_objd);
+	return event_objd;
+
+event_error:
+	{
+		int err;
+
+		err = lttng_ust_abi_objd_unref(event_objd, 1);
+		assert(!err);
+	}
+objd_error:
+	return ret;
+}
+#endif	/* CONFIG_LTTNG_UST_EXPERIMENTAL_COUNTER */
+
 /**
  *	lttng_channel_cmd - lttng control through object descriptors
  *
@@ -1240,11 +1655,13 @@ long lttng_channel_cmd(int objd, unsigned int cmd, unsigned long arg,
 			 * If the event name is a star globbing pattern,
 			 * we create the special star globbing enabler.
 			 */
-			return lttng_abi_create_event_enabler(objd, event_param,
-					owner, LTTNG_ENABLER_FORMAT_STAR_GLOB);
+			return lttng_abi_create_event_recorder_enabler(objd, lttng_chan_buf,
+					event_param, owner,
+					LTTNG_ENABLER_FORMAT_STAR_GLOB);
 		} else {
-			return lttng_abi_create_event_enabler(objd, event_param,
-					owner, LTTNG_ENABLER_FORMAT_EVENT);
+			return lttng_abi_create_event_recorder_enabler(objd, lttng_chan_buf,
+					event_param, owner,
+					LTTNG_ENABLER_FORMAT_EVENT);
 		}
 	}
 	case LTTNG_UST_ABI_CONTEXT:
@@ -1278,6 +1695,129 @@ static const struct lttng_ust_abi_objd_ops lttng_channel_ops = {
 	.cmd = lttng_channel_cmd,
 };
 
+/**
+ *	lttng_counter_cmd - lttng control through object descriptors
+ *
+ *	@objd: the object descriptor
+ *	@cmd: the command
+ *	@arg: command arg
+ *	@uargs: UST arguments (internal)
+ *	@owner: objd owner
+ *
+ *	This object descriptor implements lttng commands:
+ *      LTTNG_UST_ABI_COUNTER_CHANNEL:
+ *              Returns a channel counter object descriptor or failure.
+ *      LTTNG_UST_ABI_COUNTER_CPU:
+ *              Returns a per-cpu counter object descriptor or failure.
+ *	LTTNG_UST_ABI_COUNTER_EVENT
+ *		Returns an event object descriptor or failure.
+ *	LTTNG_UST_ABI_ENABLE
+ *		Enable recording for events in this channel (weak enable)
+ *	LTTNG_UST_ABI_DISABLE
+ *		Disable recording for events in this channel (strong disable)
+ *
+ * Counter and event object descriptors also hold a reference on the session.
+ */
+static
+long lttng_counter_cmd(int objd, unsigned int cmd, unsigned long arg,
+	union lttng_ust_abi_args *uargs, void *owner __attribute__((unused)))
+{
+	struct lttng_ust_channel_counter *counter = objd_private(objd);
+
+	if (cmd != LTTNG_UST_ABI_COUNTER_CHANNEL && cmd != LTTNG_UST_ABI_COUNTER_CPU) {
+		/*
+		 * Check if counter received all per-channel/per-cpu objects.
+		 */
+		if (!lttng_counter_ready(counter->priv->counter))
+			return -EPERM;
+	}
+
+	switch (cmd) {
+	case LTTNG_UST_ABI_COUNTER_CHANNEL:
+	{
+		struct lttng_ust_abi_counter_channel *abi_counter_channel =
+			(struct lttng_ust_abi_counter_channel *) arg;
+		struct lttng_ust_abi_counter_channel counter_channel;
+		long ret;
+		int shm_fd;
+
+		if (uargs->counter_shm.len < lttng_ust_offsetofend(struct lttng_ust_abi_counter_channel, shm_len))
+			return -EINVAL;
+		if (abi_counter_channel->len > uargs->counter_shm.len ||
+				abi_counter_channel->len < lttng_ust_offsetofend(struct lttng_ust_abi_counter_channel, shm_len)) {
+			return -EINVAL;
+		}
+		ret = copy_abi_struct(&counter_channel, sizeof(counter_channel),
+				abi_counter_channel, abi_counter_channel->len);
+		if (ret)
+			return ret;
+		shm_fd = uargs->counter_shm.shm_fd;
+		ret = lttng_counter_set_channel_shm(counter->priv->counter, shm_fd);
+		if (!ret) {
+			/* Take ownership of shm_fd. */
+			uargs->counter_shm.shm_fd = -1;
+		}
+		return ret;
+	}
+	case LTTNG_UST_ABI_COUNTER_CPU:
+	{
+		struct lttng_ust_abi_counter_cpu *abi_counter_cpu =
+			(struct lttng_ust_abi_counter_cpu *) arg;
+		struct lttng_ust_abi_counter_cpu counter_cpu;
+		long ret;
+		int shm_fd;
+
+		if (uargs->counter_shm.len < lttng_ust_offsetofend(struct lttng_ust_abi_counter_cpu, cpu_nr))
+			return -EINVAL;
+		if (abi_counter_cpu->len > uargs->counter_shm.len ||
+				abi_counter_cpu->len < lttng_ust_offsetofend(struct lttng_ust_abi_counter_cpu, cpu_nr)) {
+			return -EINVAL;
+		}
+		ret = copy_abi_struct(&counter_cpu, sizeof(counter_cpu),
+				abi_counter_cpu, abi_counter_cpu->len);
+		if (ret)
+			return ret;
+		shm_fd = uargs->counter_shm.shm_fd;
+		ret = lttng_counter_set_cpu_shm(counter->priv->counter,
+				counter_cpu.cpu_nr, shm_fd);
+		if (!ret) {
+			/* Take ownership of shm_fd. */
+			uargs->counter_shm.shm_fd = -1;
+		}
+		return ret;
+	}
+#ifdef CONFIG_LTTNG_UST_EXPERIMENTAL_COUNTER
+	case LTTNG_UST_ABI_COUNTER_EVENT:
+	{
+		return lttng_abi_create_event_counter_enabler(objd, counter,
+				arg, uargs->counter_event.len, owner);
+	}
+#endif /* CONFIG_LTTNG_UST_EXPERIMENTAL_COUNTER */
+	case LTTNG_UST_ABI_ENABLE:
+		return lttng_channel_enable(counter->parent);
+	case LTTNG_UST_ABI_DISABLE:
+		return lttng_channel_disable(counter->parent);
+	default:
+		return -EINVAL;
+	}
+}
+
+static
+int lttng_counter_release(int objd)
+{
+	struct lttng_ust_channel_counter *counter = objd_private(objd);
+
+	if (counter) {
+		return lttng_ust_abi_objd_unref(counter->parent->session->priv->objd, 0);
+	}
+	return 0;
+}
+
+static const struct lttng_ust_abi_objd_ops lttng_counter_ops = {
+	.release = lttng_counter_release,
+	.cmd = lttng_counter_cmd,
+};
+
 /**
  *	lttng_enabler_cmd - lttng control through object descriptors
  *
@@ -1305,21 +1845,21 @@ long lttng_event_enabler_cmd(int objd, unsigned int cmd, unsigned long arg,
 	union lttng_ust_abi_args *uargs __attribute__((unused)),
 	void *owner __attribute__((unused)))
 {
-	struct lttng_event_enabler *enabler = objd_private(objd);
+	struct lttng_event_enabler_session_common *enabler = objd_private(objd);
 
 	switch (cmd) {
 	case LTTNG_UST_ABI_CONTEXT:
 		return lttng_event_enabler_attach_context(enabler,
 				(struct lttng_ust_abi_context *) arg);
 	case LTTNG_UST_ABI_ENABLE:
-		return lttng_event_enabler_enable(enabler);
+		return lttng_event_enabler_enable(&enabler->parent);
 	case LTTNG_UST_ABI_DISABLE:
-		return lttng_event_enabler_disable(enabler);
+		return lttng_event_enabler_disable(&enabler->parent);
 	case LTTNG_UST_ABI_FILTER:
 	{
 		int ret;
 
-		ret = lttng_event_enabler_attach_filter_bytecode(enabler,
+		ret = lttng_event_enabler_attach_filter_bytecode(&enabler->parent,
 				(struct lttng_ust_bytecode_node **) arg);
 		if (ret)
 			return ret;
@@ -1327,7 +1867,7 @@ long lttng_event_enabler_cmd(int objd, unsigned int cmd, unsigned long arg,
 	}
 	case LTTNG_UST_ABI_EXCLUSION:
 	{
-		return lttng_event_enabler_attach_exclusion(enabler,
+		return lttng_event_enabler_attach_exclusion(&enabler->parent,
 				(struct lttng_ust_excluder_node **) arg);
 	}
 	default:
@@ -1338,7 +1878,7 @@ long lttng_event_enabler_cmd(int objd, unsigned int cmd, unsigned long arg,
 static
 int lttng_event_enabler_release(int objd)
 {
-	struct lttng_event_enabler *event_enabler = objd_private(objd);
+	struct lttng_event_recorder_enabler *event_enabler = objd_private(objd);
 
 	if (event_enabler)
 		return lttng_ust_abi_objd_unref(event_enabler->chan->priv->parent.objd, 0);
