@@ -234,10 +234,10 @@ void ust_unlock(void)
  */
 static sem_t constructor_wait;
 /*
- * Doing this for both the global and local sessiond.
+ * Doing this for the ust_app, global and local sessiond.
  */
 enum {
-	sem_count_initial_value = 4,
+	sem_count_initial_value = 6,
 };
 
 static int sem_count = sem_count_initial_value;
@@ -257,15 +257,22 @@ struct sock_info {
 	int root_handle;
 	int registration_done;
 	int allowed;
-	int global;
+	bool multi_user;
 	int thread_active;
 
 	char sock_path[PATH_MAX];
 	int socket;
 	int notify_socket;
 
+	/*
+	 * If wait_shm_is_file is true, use standard open to open and
+	 * create the shared memory used for waiting on session daemon.
+	 * Otherwise, use shm_open to create this file.
+	 */
+	bool wait_shm_is_file;
 	char wait_shm_path[PATH_MAX];
 	char *wait_shm_mmap;
+
 	/* Keep track of lazy state dump not performed yet. */
 	int statedump_pending;
 	int initial_statedump_done;
@@ -274,9 +281,29 @@ struct sock_info {
 };
 
 /* Socket from app (connect) to session daemon (listen) for communication */
+static struct sock_info ust_app = {
+	.name = "ust_app",
+	.multi_user = true,
+
+	.root_handle = -1,
+	.registration_done = 0,
+	.allowed = 0,
+	.thread_active = 0,
+
+	.socket = -1,
+	.notify_socket = -1,
+
+	.wait_shm_is_file = true,
+
+	.statedump_pending = 0,
+	.initial_statedump_done = 0,
+	.procname[0] = '\0'
+};
+
+
 static struct sock_info global_apps = {
 	.name = "global",
-	.global = 1,
+	.multi_user = true,
 
 	.root_handle = -1,
 	.registration_done = 0,
@@ -287,6 +314,7 @@ static struct sock_info global_apps = {
 	.socket = -1,
 	.notify_socket = -1,
 
+	.wait_shm_is_file = false,
 	.wait_shm_path = "/" LTTNG_UST_WAIT_FILENAME,
 
 	.statedump_pending = 0,
@@ -294,11 +322,9 @@ static struct sock_info global_apps = {
 	.procname[0] = '\0'
 };
 
-/* TODO: allow global_apps_sock_path override */
-
 static struct sock_info local_apps = {
 	.name = "local",
-	.global = 0,
+	.multi_user = false,
 	.root_handle = -1,
 	.registration_done = 0,
 	.allowed = 0,	/* Check setuid bit first */
@@ -307,6 +333,8 @@ static struct sock_info local_apps = {
 	.socket = -1,
 	.notify_socket = -1,
 
+	.wait_shm_is_file = false,
+
 	.statedump_pending = 0,
 	.initial_statedump_done = 0,
 	.procname[0] = '\0'
@@ -358,8 +386,11 @@ static const char *cmd_name_mapping[] = {
 	[ LTTNG_UST_ABI_COUNTER ] = "Create Counter",
 
 	/* Counter commands */
-	[ LTTNG_UST_ABI_COUNTER_GLOBAL ] = "Create Counter Global",
+	[ LTTNG_UST_ABI_COUNTER_CHANNEL ] = "Create Counter Channel",
 	[ LTTNG_UST_ABI_COUNTER_CPU ] = "Create Counter CPU",
+#ifdef CONFIG_LTTNG_UST_EXPERIMENTAL_COUNTER
+	[ LTTNG_UST_ABI_COUNTER_EVENT ] = "Create Counter Event",
+#endif	/* CONFIG_LTTNG_UST_EXPERIMENTAL_COUNTER */
 };
 
 static const char *str_timeout;
@@ -383,49 +414,92 @@ const char *get_lttng_home_dir(void)
        return (const char *) lttng_ust_getenv("HOME");
 }
 
+/*
+ * Returns the LTTNG_UST_APP_PATH path. If environment variable exists
+ * and contains a ':', the first path before the ':' separator is returned.
+ * The return value should be freed by the caller if it is not NULL.
+ */
+static
+char *get_lttng_ust_app_path(void)
+{
+	const char *env_val = lttng_ust_getenv("LTTNG_UST_APP_PATH");
+	char *val = NULL;
+	char *sep = NULL;
+	if (env_val == NULL)
+		goto error;
+	sep = strchr((char*)env_val, ':');
+	if (sep) {
+		/*
+		 * Split into multiple paths using ':' as a separator.
+		 * There is no escaping of the ':' separator.
+		 */
+		WARN("':' separator in LTTNG_UST_APP_PATH, only the first path will be used.");
+		val = zmalloc(sep - env_val + 1);
+		if (!val) {
+			PERROR("zmalloc get_lttng_ust_app_path");
+			goto error;
+		}
+		memcpy(val, env_val, sep - env_val);
+		val[sep - env_val] = '\0';
+	} else {
+		val = strdup(env_val);
+		if (!val) {
+			PERROR("strdup");
+			goto error;
+		}
+	}
+
+error:
+	return val;
+}
+
 /*
  * Force a read (imply TLS allocation for dlopen) of TLS variables.
  */
 static
-void lttng_nest_count_alloc_tls(void)
+void lttng_ust_nest_count_alloc_tls(void)
 {
-	asm volatile ("" : : "m" (URCU_TLS(lttng_ust_nest_count)));
+	__asm__ __volatile__ ("" : : "m" (URCU_TLS(lttng_ust_nest_count)));
 }
 
 static
 void lttng_ust_mutex_nest_alloc_tls(void)
 {
-	asm volatile ("" : : "m" (URCU_TLS(ust_mutex_nest)));
+	__asm__ __volatile__ ("" : : "m" (URCU_TLS(ust_mutex_nest)));
 }
 
 /*
  * Allocate lttng-ust urcu TLS.
  */
 static
-void lttng_lttng_ust_urcu_alloc_tls(void)
+void lttng_ust_urcu_alloc_tls(void)
 {
 	(void) lttng_ust_urcu_read_ongoing();
 }
 
-void lttng_ust_alloc_tls(void)
+void lttng_ust_common_init_thread(int flags)
 {
-	lttng_lttng_ust_urcu_alloc_tls();
+	lttng_ust_urcu_alloc_tls();
 	lttng_ringbuffer_alloc_tls();
-	lttng_vtid_alloc_tls();
-	lttng_nest_count_alloc_tls();
-	lttng_procname_alloc_tls();
+	lttng_ust_vtid_init_thread(flags);
+	lttng_ust_nest_count_alloc_tls();
+	lttng_ust_procname_init_thread(flags);
 	lttng_ust_mutex_nest_alloc_tls();
-	lttng_ust_perf_counter_alloc_tls();
+	lttng_ust_perf_counter_init_thread(flags);
 	lttng_ust_common_alloc_tls();
-	lttng_cgroup_ns_alloc_tls();
-	lttng_ipc_ns_alloc_tls();
-	lttng_net_ns_alloc_tls();
-	lttng_time_ns_alloc_tls();
-	lttng_uts_ns_alloc_tls();
+	lttng_ust_cgroup_ns_init_thread(flags);
+	lttng_ust_ipc_ns_init_thread(flags);
+	lttng_ust_net_ns_init_thread(flags);
+	lttng_ust_time_ns_init_thread(flags);
+	lttng_ust_uts_ns_init_thread(flags);
 	lttng_ust_ring_buffer_client_discard_alloc_tls();
 	lttng_ust_ring_buffer_client_discard_rt_alloc_tls();
 	lttng_ust_ring_buffer_client_overwrite_alloc_tls();
 	lttng_ust_ring_buffer_client_overwrite_rt_alloc_tls();
+	lttng_ust_ring_buffer_client_discard_per_channel_alloc_tls();
+	lttng_ust_ring_buffer_client_discard_per_channel_rt_alloc_tls();
+	lttng_ust_ring_buffer_client_overwrite_per_channel_alloc_tls();
+	lttng_ust_ring_buffer_client_overwrite_per_channel_rt_alloc_tls();
 }
 
 /*
@@ -446,7 +520,9 @@ void lttng_ust_init_thread(void)
 	 * ensure those are initialized before a signal handler nesting over
 	 * this thread attempts to use them.
 	 */
-	lttng_ust_alloc_tls();
+	lttng_ust_common_init_thread(LTTNG_UST_INIT_THREAD_MASK);
+
+	lttng_ust_urcu_register_thread();
 }
 
 int lttng_get_notify_socket(void *owner)
@@ -478,12 +554,72 @@ void print_cmd(int cmd, int handle)
 		lttng_ust_obj_get_name(handle), handle);
 }
 
+static
+int setup_ust_apps(void)
+{
+	char *ust_app_path = NULL;
+	int ret = 0;
+	uid_t uid;
+
+	assert(!ust_app.wait_shm_mmap);
+
+	uid = getuid();
+	/*
+	 * Disallow ust apps tracing for setuid binaries, because we
+	 * cannot use the environment variables anyway.
+	 */
+	if (uid != geteuid()) {
+		DBG("UST app tracing disabled for setuid binary.");
+		assert(ust_app.allowed == 0);
+		ret = 0;
+		goto end;
+	}
+	ust_app_path = get_lttng_ust_app_path();
+	if (!ust_app_path) {
+		DBG("LTTNG_UST_APP_PATH environment variable not set.");
+		assert(ust_app.allowed == 0);
+		ret = -ENOENT;
+		goto end;
+	}
+	/*
+	 * The LTTNG_UST_APP_PATH env. var. disables global and local
+	 * sessiond connections.
+	 */
+	ust_app.allowed = 1;
+	snprintf(ust_app.sock_path, PATH_MAX, "%s/%s",
+		ust_app_path, LTTNG_UST_SOCK_FILENAME);
+	snprintf(ust_app.wait_shm_path, PATH_MAX, "%s/%s",
+		ust_app_path,
+		LTTNG_UST_WAIT_FILENAME);
+
+	ust_app.wait_shm_mmap = get_map_shm(&ust_app);
+	if (!ust_app.wait_shm_mmap) {
+		WARN("Unable to get map shm for ust_app. Disabling LTTng-UST ust_app tracing.");
+		ust_app.allowed = 0;
+		ret = -EIO;
+		goto end;
+	}
+
+	lttng_pthread_getname_np(ust_app.procname, LTTNG_UST_CONTEXT_PROCNAME_LEN);
+end:
+	if (ust_app_path)
+		free(ust_app_path);
+	return ret;
+}
+
 static
 int setup_global_apps(void)
 {
 	int ret = 0;
 	assert(!global_apps.wait_shm_mmap);
 
+	/*
+	 * The LTTNG_UST_APP_PATH env. var. disables global sessiond
+	 * connections.
+	 */
+	if (ust_app.allowed)
+		return 0;
+
 	global_apps.wait_shm_mmap = get_map_shm(&global_apps);
 	if (!global_apps.wait_shm_mmap) {
 		WARN("Unable to get map shm for global apps. Disabling LTTng-UST global tracing.");
@@ -497,6 +633,7 @@ int setup_global_apps(void)
 error:
 	return ret;
 }
+
 static
 int setup_local_apps(void)
 {
@@ -506,6 +643,13 @@ int setup_local_apps(void)
 
 	assert(!local_apps.wait_shm_mmap);
 
+	/*
+	 * The LTTNG_UST_APP_PATH env. var. disables local sessiond
+	 * connections.
+	 */
+	if (ust_app.allowed)
+		return 0;
+
 	uid = getuid();
 	/*
 	 * Disallow per-user tracing for setuid binaries.
@@ -755,14 +899,53 @@ const char *bytecode_type_str(uint32_t cmd)
 {
 	switch (cmd) {
 	case LTTNG_UST_ABI_CAPTURE:
-		return "capture";
+		return "capture bytecode";
 	case LTTNG_UST_ABI_FILTER:
-		return "filter";
+		return "filter bytecode";
 	default:
 		abort();
 	}
 }
 
+enum handle_message_error {
+	MSG_OK = 0,
+	MSG_ERROR = 1,
+	MSG_SHUTDOWN = 2,
+};
+
+/*
+ * Return:
+ * < 0: error
+ * 0: OK, handle command.
+ * > 0: shutdown (no error).
+ */
+static
+enum handle_message_error handle_error(struct sock_info *sock_info, ssize_t len,
+		ssize_t expected_len, const char *str, int *error_code)
+{
+	if (!len) {
+		/* orderly shutdown */
+		*error_code = 0;
+		return MSG_SHUTDOWN;
+	}
+	if (len == expected_len) {
+		DBG("%s data received", str);
+		*error_code = 0;
+		return MSG_OK;
+	}
+	if (len < 0) {
+		DBG("Receive failed from lttng-sessiond with errno %d", (int) -len);
+		if (len == -ECONNRESET) {
+			ERR("%s remote end closed connection", sock_info->name);
+		}
+		*error_code = len;
+		return MSG_ERROR;
+	}
+	DBG("incorrect %s data message size: %zd", str, len);
+	*error_code = -EINVAL;
+	return MSG_ERROR;
+}
+
 static
 int handle_bytecode_recv(struct sock_info *sock_info,
 		int sock, struct ustcomm_ust_msg *lum)
@@ -795,14 +978,14 @@ int handle_bytecode_recv(struct sock_info *sock_info,
 	}
 
 	if (data_size > data_size_max) {
-		ERR("Bytecode %s data size is too large: %u bytes",
+		ERR("%s data size is too large: %u bytes",
 				bytecode_type_str(lum->cmd), data_size);
 		ret = -EINVAL;
 		goto end;
 	}
 
 	if (reloc_offset > data_size) {
-		ERR("Bytecode %s reloc offset %u is not within data",
+		ERR("%s reloc offset %u is not within data",
 				bytecode_type_str(lum->cmd), reloc_offset);
 		ret = -EINVAL;
 		goto end;
@@ -821,34 +1004,13 @@ int handle_bytecode_recv(struct sock_info *sock_info,
 	bytecode->type = type;
 
 	len = ustcomm_recv_unix_sock(sock, bytecode->bc.data, bytecode->bc.len);
-	switch (len) {
-	case 0:	/* orderly shutdown */
-		ret = 0;
+	switch (handle_error(sock_info, len, bytecode->bc.len, bytecode_type_str(lum->cmd), &ret)) {
+	case MSG_OK:
+		break;
+	case MSG_ERROR:		/* Fallthrough */
+	case MSG_SHUTDOWN:
 		goto end;
-	default:
-		if (len == bytecode->bc.len) {
-			DBG("Bytecode %s data received",
-					bytecode_type_str(lum->cmd));
-			break;
-		} else if (len < 0) {
-			DBG("Receive failed from lttng-sessiond with errno %d",
-					(int) -len);
-			if (len == -ECONNRESET) {
-				ERR("%s remote end closed connection",
-						sock_info->name);
-				ret = len;
-				goto end;
-			}
-			ret = len;
-			goto end;
-		} else {
-			DBG("Incorrect %s bytecode data message size: %zd",
-					bytecode_type_str(lum->cmd), len);
-			ret = -EINVAL;
-			goto end;
-		}
 	}
-
 	ops = lttng_ust_abi_objd_ops(lum->handle);
 	if (!ops) {
 		ret = -ENOENT;
@@ -916,12 +1078,11 @@ int handle_message(struct sock_info *sock_info,
 {
 	int ret = 0;
 	const struct lttng_ust_abi_objd_ops *ops;
-	struct ustcomm_ust_reply lur;
+	struct ustcomm_ust_reply lur = {};
 	union lttng_ust_abi_args args;
 	char ctxstr[LTTNG_UST_ABI_SYM_NAME_LEN];	/* App context string. */
 	ssize_t len;
-
-	memset(&lur, 0, sizeof(lur));
+	void *var_len_cmd_data = NULL;
 
 	if (ust_lock()) {
 		ret = -LTTNG_UST_ERR_EXITING;
@@ -955,8 +1116,11 @@ int handle_message(struct sock_info *sock_info,
 
 	case LTTNG_UST_ABI_CAPTURE:
 	case LTTNG_UST_ABI_COUNTER:
-	case LTTNG_UST_ABI_COUNTER_GLOBAL:
+	case LTTNG_UST_ABI_COUNTER_CHANNEL:
 	case LTTNG_UST_ABI_COUNTER_CPU:
+#ifdef CONFIG_LTTNG_UST_EXPERIMENTAL_COUNTER
+	case LTTNG_UST_ABI_COUNTER_EVENT:
+#endif	/* CONFIG_LTTNG_UST_EXPERIMENTAL_COUNTER */
 	case LTTNG_UST_ABI_EVENT_NOTIFIER_CREATE:
 	case LTTNG_UST_ABI_EVENT_NOTIFIER_GROUP_CREATE:
 		/*
@@ -1020,32 +1184,13 @@ int handle_message(struct sock_info *sock_info,
 		node->excluder.count = count;
 		len = ustcomm_recv_unix_sock(sock, node->excluder.names,
 				count * LTTNG_UST_ABI_SYM_NAME_LEN);
-		switch (len) {
-		case 0:	/* orderly shutdown */
-			ret = 0;
+		switch (handle_error(sock_info, len, count * LTTNG_UST_ABI_SYM_NAME_LEN, "exclusion", &ret)) {
+		case MSG_OK:
+			break;
+		case MSG_ERROR:		/* Fallthrough */
+		case MSG_SHUTDOWN:
 			free(node);
 			goto error;
-		default:
-			if (len == count * LTTNG_UST_ABI_SYM_NAME_LEN) {
-				DBG("Exclusion data received");
-				break;
-			} else if (len < 0) {
-				DBG("Receive failed from lttng-sessiond with errno %d", (int) -len);
-				if (len == -ECONNRESET) {
-					ERR("%s remote end closed connection", sock_info->name);
-					ret = len;
-					free(node);
-					goto error;
-				}
-				ret = len;
-				free(node);
-				goto error;
-			} else {
-				DBG("Incorrect exclusion data message size: %zd", len);
-				ret = -EINVAL;
-				free(node);
-				goto error;
-			}
 		}
 		if (ops->cmd)
 			ret = ops->cmd(lum->handle, lum->cmd,
@@ -1062,33 +1207,14 @@ int handle_message(struct sock_info *sock_info,
 
 		len = ustcomm_recv_event_notifier_notif_fd_from_sessiond(sock,
 			&event_notifier_notif_fd);
-		switch (len) {
-		case 0:	/* orderly shutdown */
-			ret = 0;
-			goto error;
-		case 1:
+		switch (handle_error(sock_info, len, 1, "event notifier group", &ret)) {
+		case MSG_OK:
 			break;
-		default:
-			if (len < 0) {
-				DBG("Receive failed from lttng-sessiond with errno %d",
-						(int) -len);
-				if (len == -ECONNRESET) {
-					ERR("%s remote end closed connection",
-							sock_info->name);
-					ret = len;
-					goto error;
-				}
-				ret = len;
-				goto error;
-			} else {
-				DBG("Incorrect event notifier fd message size: %zd",
-						len);
-				ret = -EINVAL;
-				goto error;
-			}
+		case MSG_ERROR:		/* Fallthrough */
+		case MSG_SHUTDOWN:
+			goto error;
 		}
-		args.event_notifier_handle.event_notifier_notif_fd =
-				event_notifier_notif_fd;
+		args.event_notifier_handle.event_notifier_notif_fd = event_notifier_notif_fd;
 		if (ops->cmd)
 			ret = ops->cmd(lum->handle, lum->cmd,
 					(unsigned long) &lum->u,
@@ -1112,28 +1238,12 @@ int handle_message(struct sock_info *sock_info,
 		len = ustcomm_recv_channel_from_sessiond(sock,
 				&chan_data, lum->u.channel.len,
 				&wakeup_fd);
-		switch (len) {
-		case 0:	/* orderly shutdown */
-			ret = 0;
+		switch (handle_error(sock_info, len, lum->u.channel.len, "channel", &ret)) {
+		case MSG_OK:
+			break;
+		case MSG_ERROR:		/* Fallthrough */
+		case MSG_SHUTDOWN:
 			goto error;
-		default:
-			if (len == lum->u.channel.len) {
-				DBG("channel data received");
-				break;
-			} else if (len < 0) {
-				DBG("Receive failed from lttng-sessiond with errno %d", (int) -len);
-				if (len == -ECONNRESET) {
-					ERR("%s remote end closed connection", sock_info->name);
-					ret = len;
-					goto error;
-				}
-				ret = len;
-				goto error;
-			} else {
-				DBG("incorrect channel data message size: %zd", len);
-				ret = -EINVAL;
-				goto error;
-			}
 		}
 		args.channel.chan_data = chan_data;
 		args.channel.wakeup_fd = wakeup_fd;
@@ -1212,28 +1322,12 @@ int handle_message(struct sock_info *sock_info,
 			p = &ctxstr[strlen("$app.")];
 			recvlen = ctxlen - strlen("$app.");
 			len = ustcomm_recv_unix_sock(sock, p, recvlen);
-			switch (len) {
-			case 0:	/* orderly shutdown */
-				ret = 0;
+			switch (handle_error(sock_info, len, recvlen, "app context", &ret)) {
+			case MSG_OK:
+				break;
+			case MSG_ERROR:		/* Fallthrough */
+			case MSG_SHUTDOWN:
 				goto error;
-			default:
-				if (len == recvlen) {
-					DBG("app context data received");
-					break;
-				} else if (len < 0) {
-					DBG("Receive failed from lttng-sessiond with errno %d", (int) -len);
-					if (len == -ECONNRESET) {
-						ERR("%s remote end closed connection", sock_info->name);
-						ret = len;
-						goto error;
-					}
-					ret = len;
-					goto error;
-				} else {
-					DBG("incorrect app context data message size: %zd", len);
-					ret = -EINVAL;
-					goto error;
-				}
 			}
 			/* Put : between provider and ctxname. */
 			p[lum->u.context.u.app_ctx.provider_name_len - 1] = ':';
@@ -1253,55 +1347,44 @@ int handle_message(struct sock_info *sock_info,
 		break;
 	case LTTNG_UST_ABI_COUNTER:
 	{
-		void *counter_data;
-
-		len = ustcomm_recv_counter_from_sessiond(sock,
-				&counter_data, lum->u.counter.len);
-		switch (len) {
-		case 0:	/* orderly shutdown */
-			ret = 0;
+		len = ustcomm_recv_var_len_cmd_from_sessiond(sock,
+				&var_len_cmd_data, lum->u.var_len_cmd.cmd_len);
+		switch (handle_error(sock_info, len, lum->u.var_len_cmd.cmd_len, "counter", &ret)) {
+		case MSG_OK:
+			break;
+		case MSG_ERROR:		/* Fallthrough */
+		case MSG_SHUTDOWN:
 			goto error;
-		default:
-			if (len == lum->u.counter.len) {
-				DBG("counter data received");
-				break;
-			} else if (len < 0) {
-				DBG("Receive failed from lttng-sessiond with errno %d", (int) -len);
-				if (len == -ECONNRESET) {
-					ERR("%s remote end closed connection", sock_info->name);
-					ret = len;
-					goto error;
-				}
-				ret = len;
-				goto error;
-			} else {
-				DBG("incorrect counter data message size: %zd", len);
-				ret = -EINVAL;
-				goto error;
-			}
 		}
-		args.counter.counter_data = counter_data;
+		args.counter.len = lum->u.var_len_cmd.cmd_len;
 		if (ops->cmd)
 			ret = ops->cmd(lum->handle, lum->cmd,
-					(unsigned long) &lum->u,
+					(unsigned long) var_len_cmd_data,
 					&args, sock_info);
 		else
 			ret = -ENOSYS;
-		free(args.counter.counter_data);
 		break;
 	}
-	case LTTNG_UST_ABI_COUNTER_GLOBAL:
+	case LTTNG_UST_ABI_COUNTER_CHANNEL:
 	{
+		len = ustcomm_recv_var_len_cmd_from_sessiond(sock,
+				&var_len_cmd_data, lum->u.var_len_cmd.cmd_len);
+		switch (handle_error(sock_info, len, lum->u.var_len_cmd.cmd_len, "counter channel", &ret)) {
+		case MSG_OK:
+			break;
+		case MSG_ERROR:		/* Fallthrough */
+		case MSG_SHUTDOWN:
+			goto error;
+		}
 		/* Receive shm_fd */
-		ret = ustcomm_recv_counter_shm_from_sessiond(sock,
-			&args.counter_shm.shm_fd);
+		ret = ustcomm_recv_counter_shm_from_sessiond(sock, &args.counter_shm.shm_fd);
 		if (ret) {
 			goto error;
 		}
-
+		args.counter_shm.len = lum->u.var_len_cmd.cmd_len;
 		if (ops->cmd)
 			ret = ops->cmd(lum->handle, lum->cmd,
-					(unsigned long) &lum->u,
+					(unsigned long) var_len_cmd_data,
 					&args, sock_info);
 		else
 			ret = -ENOSYS;
@@ -1319,16 +1402,24 @@ int handle_message(struct sock_info *sock_info,
 	}
 	case LTTNG_UST_ABI_COUNTER_CPU:
 	{
+		len = ustcomm_recv_var_len_cmd_from_sessiond(sock,
+				&var_len_cmd_data, lum->u.var_len_cmd.cmd_len);
+		switch (handle_error(sock_info, len, lum->u.var_len_cmd.cmd_len, "counter cpu", &ret)) {
+		case MSG_OK:
+			break;
+		case MSG_ERROR:		/* Fallthrough */
+		case MSG_SHUTDOWN:
+			goto error;
+		}
 		/* Receive shm_fd */
-		ret = ustcomm_recv_counter_shm_from_sessiond(sock,
-			&args.counter_shm.shm_fd);
+		ret = ustcomm_recv_counter_shm_from_sessiond(sock, &args.counter_shm.shm_fd);
 		if (ret) {
 			goto error;
 		}
-
+		args.counter_shm.len = lum->u.var_len_cmd.cmd_len;
 		if (ops->cmd)
 			ret = ops->cmd(lum->handle, lum->cmd,
-					(unsigned long) &lum->u,
+					(unsigned long) var_len_cmd_data,
 					&args, sock_info);
 		else
 			ret = -ENOSYS;
@@ -1344,43 +1435,43 @@ int handle_message(struct sock_info *sock_info,
 		}
 		break;
 	}
-	case LTTNG_UST_ABI_EVENT_NOTIFIER_CREATE:
+#ifdef CONFIG_LTTNG_UST_EXPERIMENTAL_COUNTER
+	case LTTNG_UST_ABI_COUNTER_EVENT:
 	{
-		/* Receive struct lttng_ust_event_notifier */
-		struct lttng_ust_abi_event_notifier event_notifier;
-
-		if (sizeof(event_notifier) != lum->u.event_notifier.len) {
-			DBG("incorrect event notifier data message size: %u", lum->u.event_notifier.len);
-			ret = -EINVAL;
+		len = ustcomm_recv_var_len_cmd_from_sessiond(sock,
+				&var_len_cmd_data, lum->u.var_len_cmd.cmd_len);
+		switch (handle_error(sock_info, len, lum->u.var_len_cmd.cmd_len, "counter event", &ret)) {
+		case MSG_OK:
+			break;
+		case MSG_ERROR:		/* Fallthrough */
+		case MSG_SHUTDOWN:
 			goto error;
 		}
-		len = ustcomm_recv_unix_sock(sock, &event_notifier, sizeof(event_notifier));
-		switch (len) {
-		case 0:	/* orderly shutdown */
-			ret = 0;
-			goto error;
-		default:
-			if (len == sizeof(event_notifier)) {
-				DBG("event notifier data received");
-				break;
-			} else if (len < 0) {
-				DBG("Receive failed from lttng-sessiond with errno %d", (int) -len);
-				if (len == -ECONNRESET) {
-					ERR("%s remote end closed connection", sock_info->name);
-					ret = len;
-					goto error;
-				}
-				ret = len;
-				goto error;
-			} else {
-				DBG("incorrect event notifier data message size: %zd", len);
-				ret = -EINVAL;
-				goto error;
-			}
-		}
+		args.counter_event.len = lum->u.var_len_cmd.cmd_len;
 		if (ops->cmd)
 			ret = ops->cmd(lum->handle, lum->cmd,
-					(unsigned long) &event_notifier,
+					(unsigned long) var_len_cmd_data,
+					&args, sock_info);
+		else
+			ret = -ENOSYS;
+		break;
+	}
+#endif	/* CONFIG_LTTNG_UST_EXPERIMENTAL_COUNTER */
+	case LTTNG_UST_ABI_EVENT_NOTIFIER_CREATE:
+	{
+		len = ustcomm_recv_var_len_cmd_from_sessiond(sock,
+				&var_len_cmd_data, lum->u.var_len_cmd.cmd_len);
+		switch (handle_error(sock_info, len, lum->u.var_len_cmd.cmd_len, "event notifier", &ret)) {
+		case MSG_OK:
+			break;
+		case MSG_ERROR:		/* Fallthrough */
+		case MSG_SHUTDOWN:
+			goto error;
+		}
+		args.event_notifier.len = lum->u.var_len_cmd.cmd_len;
+		if (ops->cmd)
+			ret = ops->cmd(lum->handle, lum->cmd,
+					(unsigned long) var_len_cmd_data,
 					&args, sock_info);
 		else
 			ret = -ENOSYS;
@@ -1455,6 +1546,7 @@ int handle_message(struct sock_info *sock_info,
 error:
 	ust_unlock();
 
+	free(var_len_cmd_data);
 	return ret;
 }
 
@@ -1518,13 +1610,22 @@ void cleanup_sock_info(struct sock_info *sock_info, int exiting)
 	}
 }
 
+static
+int wait_shm_open(struct sock_info *sock_info, int flags, mode_t mode)
+{
+	if (sock_info->wait_shm_is_file)
+		return open(sock_info->wait_shm_path, flags, mode);
+	else
+		return shm_open(sock_info->wait_shm_path, flags, mode);
+}
+
 /*
  * Using fork to set umask in the child process (not multi-thread safe).
  * We deal with the shm_open vs ftruncate race (happening when the
  * sessiond owns the shm and does not let everybody modify it, to ensure
  * safety against shm_unlink) by simply letting the mmap fail and
  * retrying after a few seconds.
- * For global shm, everybody has rw access to it until the sessiond
+ * For channel shm, everybody has rw access to it until the sessiond
  * starts.
  */
 static
@@ -1536,7 +1637,7 @@ int get_wait_shm(struct sock_info *sock_info, size_t mmap_size)
 	/*
 	 * Try to open read-only.
 	 */
-	wait_shm_fd = shm_open(sock_info->wait_shm_path, O_RDONLY, 0);
+	wait_shm_fd = wait_shm_open(sock_info, O_RDONLY, 0);
 	if (wait_shm_fd >= 0) {
 		int32_t tmp_read;
 		ssize_t len;
@@ -1582,21 +1683,21 @@ open_write:
 	pid = fork();
 	URCU_TLS(lttng_ust_nest_count)--;
 	if (pid > 0) {
-		int status;
+		int status, wait_ret;
 
 		/*
 		 * Parent: wait for child to return, in which case the
 		 * shared memory map will have been created.
 		 */
-		pid = wait(&status);
-		if (pid < 0 || !WIFEXITED(status) || WEXITSTATUS(status) != 0) {
+		wait_ret = waitpid(pid, &status, 0);
+		if (wait_ret < 0 || !WIFEXITED(status) || WEXITSTATUS(status) != 0) {
 			wait_shm_fd = -1;
 			goto end;
 		}
 		/*
 		 * Try to open read-only again after creation.
 		 */
-		wait_shm_fd = shm_open(sock_info->wait_shm_path, O_RDONLY, 0);
+		wait_shm_fd = wait_shm_open(sock_info, O_RDONLY, 0);
 		if (wait_shm_fd < 0) {
 			/*
 			 * Real-only open did not work. It's a failure
@@ -1611,7 +1712,7 @@ open_write:
 
 		/* Child */
 		create_mode = S_IRUSR | S_IWUSR | S_IRGRP;
-		if (sock_info->global)
+		if (sock_info->multi_user)
 			create_mode |= S_IROTH | S_IWGRP | S_IWOTH;
 		/*
 		 * We're alone in a child process, so we can modify the
@@ -1623,7 +1724,7 @@ open_write:
 		 * We don't do an exclusive open, because we allow other
 		 * processes to create+ftruncate it concurrently.
 		 */
-		wait_shm_fd = shm_open(sock_info->wait_shm_path,
+		wait_shm_fd = wait_shm_open(sock_info,
 				O_RDWR | O_CREAT, create_mode);
 		if (wait_shm_fd >= 0) {
 			ret = ftruncate(wait_shm_fd, mmap_size);
@@ -1636,12 +1737,12 @@ open_write:
 		/*
 		 * For local shm, we need to have rw access to accept
 		 * opening it: this means the local sessiond will be
-		 * able to wake us up. For global shm, we open it even
+		 * able to wake us up. For channel shm, we open it even
 		 * if rw access is not granted, because the root.root
 		 * sessiond will be able to override all rights and wake
 		 * us up.
 		 */
-		if (!sock_info->global && errno != EACCES) {
+		if (!sock_info->multi_user && errno != EACCES) {
 			ERR("Error opening shm %s", sock_info->wait_shm_path);
 			_exit(EXIT_FAILURE);
 		}
@@ -1654,7 +1755,7 @@ open_write:
 		return -1;
 	}
 end:
-	if (wait_shm_fd >= 0 && !sock_info->global) {
+	if (wait_shm_fd >= 0 && !sock_info->multi_user) {
 		struct stat statbuf;
 
 		/*
@@ -1815,7 +1916,7 @@ void *ust_listener_thread(void *arg)
 	int sock, ret, prev_connect_failed = 0, has_waited = 0, fd;
 	long timeout;
 
-	lttng_ust_alloc_tls();
+	lttng_ust_common_init_thread(0);
 	/*
 	 * If available, add '-ust' to the end of this thread's
 	 * process name
@@ -2183,7 +2284,7 @@ void lttng_ust_ctor(void)
 	 * to be the dynamic linker mutex) and ust_lock, taken within
 	 * the ust lock.
 	 */
-	lttng_ust_alloc_tls();
+	lttng_ust_common_init_thread(0);
 
 	lttng_ust_loaded = 1;
 
@@ -2203,7 +2304,7 @@ void lttng_ust_ctor(void)
 	 * this library so it never becomes zero, thus never gets unloaded from the
 	 * address space of the process. Since we are already running in the
 	 * constructor of the LTTNG_UST_LIB_SONAME library, calling dlopen will
-	 * simply increment the refcount and no additionnal work is needed by the
+	 * simply increment the refcount and no additional work is needed by the
 	 * dynamic loader as the shared library is already loaded in the address
 	 * space. As a safe guard, we use the RTLD_NODELETE flag to prevent
 	 * unloading of the UST library if its refcount becomes zero (which should
@@ -2256,6 +2357,11 @@ void lttng_ust_ctor(void)
 		PERROR("sem_init");
 	}
 
+	ret = setup_ust_apps();
+	if (ret) {
+		assert(ust_app.allowed == 0);
+		DBG("ust_app setup returned %d", ret);
+	}
 	ret = setup_global_apps();
 	if (ret) {
 		assert(global_apps.allowed == 0);
@@ -2288,6 +2394,19 @@ void lttng_ust_ctor(void)
 		ERR("pthread_attr_setdetachstate: %s", strerror(ret));
 	}
 
+	if (ust_app.allowed) {
+		pthread_mutex_lock(&ust_exit_mutex);
+		ret = pthread_create(&ust_app.ust_listener, &thread_attr,
+				ust_listener_thread, &ust_app);
+		if (ret) {
+			ERR("pthread_create ust_app: %s", strerror(ret));
+		}
+		ust_app.thread_active = 1;
+		pthread_mutex_unlock(&ust_exit_mutex);
+	} else {
+		handle_register_done(&ust_app);
+	}
+
 	if (global_apps.allowed) {
 		pthread_mutex_lock(&ust_exit_mutex);
 		ret = pthread_create(&global_apps.ust_listener, &thread_attr,
@@ -2367,8 +2486,10 @@ void lttng_ust_ctor(void)
 static
 void lttng_ust_cleanup(int exiting)
 {
+	cleanup_sock_info(&ust_app, exiting);
 	cleanup_sock_info(&global_apps, exiting);
 	cleanup_sock_info(&local_apps, exiting);
+	ust_app.allowed = 0;
 	local_apps.allowed = 0;
 	global_apps.allowed = 0;
 	/*
@@ -2418,6 +2539,15 @@ void lttng_ust_exit(void)
 
 	pthread_mutex_lock(&ust_exit_mutex);
 	/* cancel threads */
+	if (ust_app.thread_active) {
+		ret = pthread_cancel(ust_app.ust_listener);
+		if (ret) {
+			ERR("Error cancelling ust listener thread: %s",
+				strerror(ret));
+		} else {
+			ust_app.thread_active = 0;
+		}
+	}
 	if (global_apps.thread_active) {
 		ret = pthread_cancel(global_apps.ust_listener);
 		if (ret) {
@@ -2497,7 +2627,7 @@ void lttng_ust_before_fork(sigset_t *save_sigset)
 	int ret;
 
 	/* Allocate lttng-ust TLS. */
-	lttng_ust_alloc_tls();
+	lttng_ust_common_init_thread(0);
 
 	if (URCU_TLS(lttng_ust_nest_count))
 		return;
@@ -2540,7 +2670,7 @@ void lttng_ust_after_fork_parent(sigset_t *restore_sigset)
 		return;
 	DBG("process %d", getpid());
 	lttng_ust_urcu_after_fork_parent();
-	/* Release mutexes and reenable signals */
+	/* Release mutexes and re-enable signals */
 	ust_after_fork_common(restore_sigset);
 }
 
@@ -2567,7 +2697,7 @@ void lttng_ust_after_fork_child(sigset_t *restore_sigset)
 	/* Release urcu mutexes */
 	lttng_ust_urcu_after_fork_child();
 	lttng_ust_cleanup(0);
-	/* Release mutexes and reenable signals */
+	/* Release mutexes and re-enable signals */
 	ust_after_fork_common(restore_sigset);
 	lttng_ust_ctor();
 }
